@@ -2,24 +2,23 @@
 # Copyright: (c) 2017, Ansible Project
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-# Make coding more python3-ish
-from __future__ import (absolute_import, division, print_function)
-__metaclass__ = type
+from __future__ import annotations
 
 import copy
 import os
 import os.path
 import re
 import tempfile
+import typing as t
 
 from ansible import constants as C
 from ansible.errors import AnsibleFileNotFound, AnsibleParserError
 from ansible.module_utils.basic import is_executable
 from ansible.module_utils.six import binary_type, text_type
-from ansible.module_utils._text import to_bytes, to_native, to_text
+from ansible.module_utils.common.text.converters import to_bytes, to_native, to_text
 from ansible.parsing.quoting import unquote
 from ansible.parsing.utils.yaml import from_yaml
-from ansible.parsing.vault import VaultLib, b_HEADER, is_encrypted, is_encrypted_file, parse_vaulttext_envelope
+from ansible.parsing.vault import VaultLib, b_HEADER, is_encrypted, is_encrypted_file, parse_vaulttext_envelope, PromptVaultSecret
 from ansible.utils.path import unfrackpath
 from ansible.utils.display import Display
 
@@ -45,7 +44,7 @@ class DataLoader:
     Usage:
 
         dl = DataLoader()
-        # optionally: dl.set_vault_password('foo')
+        # optionally: dl.set_vault_secrets([('default', ansible.parsing.vault.PrompVaultSecret(...),)])
         ds = dl.load('...')
         ds = dl.load_from_file('/path/to/file')
     '''
@@ -66,67 +65,79 @@ class DataLoader:
         # initialize the vault stuff with an empty password
         # TODO: replace with a ref to something that can get the password
         #       a creds/auth provider
-        # self.set_vault_password(None)
         self._vaults = {}
         self._vault = VaultLib()
         self.set_vault_secrets(None)
 
     # TODO: since we can query vault_secrets late, we could provide this to DataLoader init
-    def set_vault_secrets(self, vault_secrets):
+    def set_vault_secrets(self, vault_secrets: list[tuple[str, PromptVaultSecret]] | None) -> None:
         self._vault.secrets = vault_secrets
 
-    def load(self, data, file_name='<string>', show_content=True, json_only=False):
+    def load(self, data: str, file_name: str = '<string>', show_content: bool = True, json_only: bool = False) -> t.Any:
         '''Backwards compat for now'''
         return from_yaml(data, file_name, show_content, self._vault.secrets, json_only=json_only)
 
-    def load_from_file(self, file_name, cache=True, unsafe=False, json_only=False):
-        ''' Loads data from a file, which can contain either JSON or YAML.  '''
+    def load_from_file(self, file_name: str, cache: str = 'all', unsafe: bool = False, json_only: bool = False) -> t.Any:
+        '''
+        Loads data from a file, which can contain either JSON or YAML.
 
+        :param file_name: The name of the file to load data from.
+        :param cache: Options for caching: none|all|vaulted
+        :param unsafe: If True, returns the parsed data as-is without deep copying.
+        :param json_only: If True, only loads JSON data from the file.
+        :return: The loaded data, optionally deep-copied for safety.
+        '''
+
+        # Resolve the file name
         file_name = self.path_dwim(file_name)
+
+        # Log the file being loaded
         display.debug("Loading data from %s" % file_name)
 
-        # if the file has already been read in and cached, we'll
-        # return those results to avoid more file/vault operations
-        if cache and file_name in self._FILE_CACHE:
+        # Check if the file has been cached and use the cached data if available
+        if cache != 'none' and file_name in self._FILE_CACHE:
             parsed_data = self._FILE_CACHE[file_name]
         else:
-            # read the file contents and load the data structure from them
+            # Read the file contents and load the data structure from them
             (b_file_data, show_content) = self._get_file_contents(file_name)
 
             file_data = to_text(b_file_data, errors='surrogate_or_strict')
             parsed_data = self.load(data=file_data, file_name=file_name, show_content=show_content, json_only=json_only)
 
-            # cache the file contents for next time
-            self._FILE_CACHE[file_name] = parsed_data
+            # Cache the file contents for next time based on the cache option
+            if cache == 'all':
+                self._FILE_CACHE[file_name] = parsed_data
+            elif cache == 'vaulted' and not show_content:
+                self._FILE_CACHE[file_name] = parsed_data
 
+        # Return the parsed data, optionally deep-copied for safety
         if unsafe:
             return parsed_data
         else:
-            # return a deep copy here, so the cache is not affected
             return copy.deepcopy(parsed_data)
 
-    def path_exists(self, path):
+    def path_exists(self, path: str) -> bool:
         path = self.path_dwim(path)
         return os.path.exists(to_bytes(path, errors='surrogate_or_strict'))
 
-    def is_file(self, path):
+    def is_file(self, path: str) -> bool:
         path = self.path_dwim(path)
         return os.path.isfile(to_bytes(path, errors='surrogate_or_strict')) or path == os.devnull
 
-    def is_directory(self, path):
+    def is_directory(self, path: str) -> bool:
         path = self.path_dwim(path)
         return os.path.isdir(to_bytes(path, errors='surrogate_or_strict'))
 
-    def list_directory(self, path):
+    def list_directory(self, path: str) -> list[str]:
         path = self.path_dwim(path)
         return os.listdir(path)
 
-    def is_executable(self, path):
+    def is_executable(self, path: str) -> bool:
         '''is the given path executable?'''
         path = self.path_dwim(path)
         return is_executable(path)
 
-    def _decrypt_if_vault_data(self, b_vault_data, b_file_name=None):
+    def _decrypt_if_vault_data(self, b_vault_data: bytes, b_file_name: bytes | None = None) -> tuple[bytes, bool]:
         '''Decrypt b_vault_data if encrypted and return b_data and the show_content flag'''
 
         if not is_encrypted(b_vault_data):
@@ -139,7 +150,7 @@ class DataLoader:
         show_content = False
         return b_data, show_content
 
-    def _get_file_contents(self, file_name):
+    def _get_file_contents(self, file_name: str) -> tuple[bytes, bool]:
         '''
         Reads the file contents from the given file name
 
@@ -168,17 +179,17 @@ class DataLoader:
         except (IOError, OSError) as e:
             raise AnsibleParserError("an error occurred while trying to read the file '%s': %s" % (file_name, to_native(e)), orig_exc=e)
 
-    def get_basedir(self):
+    def get_basedir(self) -> str:
         ''' returns the current basedir '''
         return self._basedir
 
-    def set_basedir(self, basedir):
+    def set_basedir(self, basedir: str) -> None:
         ''' sets the base directory, used to find files when a relative path is given '''
 
         if basedir is not None:
             self._basedir = to_text(basedir)
 
-    def path_dwim(self, given):
+    def path_dwim(self, given: str) -> str:
         '''
         make relative paths work like folks expect.
         '''
@@ -194,7 +205,7 @@ class DataLoader:
 
         return unfrackpath(path, follow=False)
 
-    def _is_role(self, path):
+    def _is_role(self, path: str) -> bool:
         ''' imperfect role detection, roles are still valid w/o tasks|meta/main.yml|yaml|etc '''
 
         b_path = to_bytes(path, errors='surrogate_or_strict')
@@ -228,7 +239,7 @@ class DataLoader:
 
         return False
 
-    def path_dwim_relative(self, path, dirname, source, is_role=False):
+    def path_dwim_relative(self, path: str, dirname: str, source: str, is_role: bool = False) -> str:
         '''
         find one file in either a role or playbook dir with or without
         explicitly named dirname subdirs
@@ -283,7 +294,7 @@ class DataLoader:
 
         return candidate
 
-    def path_dwim_relative_stack(self, paths, dirname, source, is_role=False):
+    def path_dwim_relative_stack(self, paths: list[str], dirname: str, source: str, is_role: bool = False) -> str:
         '''
         find one file in first path in stack taking roles into account and adding play basedir as fallback
 
@@ -342,7 +353,7 @@ class DataLoader:
 
         return result
 
-    def _create_content_tempfile(self, content):
+    def _create_content_tempfile(self, content: str | bytes) -> str:
         ''' Create a tempfile containing defined content '''
         fd, content_tempfile = tempfile.mkstemp(dir=C.DEFAULT_LOCAL_TMP)
         f = os.fdopen(fd, 'wb')
@@ -356,7 +367,7 @@ class DataLoader:
             f.close()
         return content_tempfile
 
-    def get_real_file(self, file_path, decrypt=True):
+    def get_real_file(self, file_path: str, decrypt: bool = True) -> str:
         """
         If the file is vault encrypted return a path to a temporary decrypted file
         If the file is not encrypted then the path is returned
@@ -396,7 +407,7 @@ class DataLoader:
         except (IOError, OSError) as e:
             raise AnsibleParserError("an error occurred while trying to read the file '%s': %s" % (to_native(real_path), to_native(e)), orig_exc=e)
 
-    def cleanup_tmp_file(self, file_path):
+    def cleanup_tmp_file(self, file_path: str) -> None:
         """
         Removes any temporary files created from a previous call to
         get_real_file. file_path must be the path returned from a
@@ -406,18 +417,18 @@ class DataLoader:
             os.unlink(file_path)
             self._tempfiles.remove(file_path)
 
-    def cleanup_all_tmp_files(self):
+    def cleanup_all_tmp_files(self) -> None:
         """
         Removes all temporary files that DataLoader has created
         NOTE: not thread safe, forks also need special handling see __init__ for details.
         """
-        for f in self._tempfiles:
+        for f in list(self._tempfiles):
             try:
                 self.cleanup_tmp_file(f)
             except Exception as e:
                 display.warning("Unable to cleanup temp files: %s" % to_text(e))
 
-    def find_vars_files(self, path, name, extensions=None, allow_dir=True):
+    def find_vars_files(self, path: str, name: str, extensions: list[str] | None = None, allow_dir: bool = True) -> list[str]:
         """
         Find vars files in a given path with specified name. This will find
         files in a dir named <name>/ or a file called <name> ending in known
@@ -447,11 +458,11 @@ class DataLoader:
                     else:
                         continue
                 else:
-                    found.append(full_path)
+                    found.append(to_text(full_path))
                 break
         return found
 
-    def _get_dir_vars_files(self, path, extensions):
+    def _get_dir_vars_files(self, path: str, extensions: list[str]) -> list[str]:
         found = []
         for spath in sorted(self.list_directory(path)):
             if not spath.startswith(u'.') and not spath.endswith(u'~'):  # skip hidden and backups
